//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Input_Library v1.4
//  Example5- Force Feedback Joysticks
//
// Date: 17th May, 2001
// Author: Darren John Dwyer
// Copyright: (c) 2001 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to control a Force Feedback Joystick,
// (or other Force Feedback Device)
//
// It helps significantly if you view and understand the previous DirectInput
// examples before you try to understand this demo.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 or 4 only
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Input"
#pragma link "TDx_InputDevice"
#pragma link "TDIDevCaps"
#pragma link "tdx_inputeffect"
#pragma link "tdidataformat"
#pragma link "tdijoystate2"
#pragma link "tdipropdword"
#pragma link "tdipropheader"
#pragma link "tdiperiodic"
#pragma link "tdienvelope"
#pragma link "tdieffect"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Input_Library_Install"
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Width=780;
Height=580;
Position=poScreenCenter;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Input1->Created) Dx_Input1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumerateButtonClick(TObject *Sender)
{
Msg("Scanning for Available Devices");

NumDevices = 0;
Dx_Input1->EnumDevices(DIDEVTYPE_JOYSTICK, NULL, DIEDFL_FORCEFEEDBACK | DIEDFL_ATTACHEDONLY );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearMessagesButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1EnumDevices(TObject *Sender,
    TDIDeviceInstance *DeviceInstance, void *Context, bool &Finished)
{
if (NumDevices<MAX_DEVICES)
    {
    DevTypes[NumDevices] = DeviceInstance->DevType;
    FFDriverGuids[NumDevices] = DeviceInstance->FFDriver;
    InstanceGuids[NumDevices] = DeviceInstance->Instance;
    InstanceNames[NumDevices] = DeviceInstance->InstanceName;
    ProductGuids[NumDevices] = DeviceInstance->Product;
    ProductNames[NumDevices] = DeviceInstance->ProductName;
    Usages[NumDevices] = DeviceInstance->Usage;
    UsagePages[NumDevices] = DeviceInstance->UsagePage;
    NumDevices++;

    Msg("Found a device: "+DeviceInstance->ProductName);
    DevicesListBox->Items->Add( DeviceInstance->ProductName );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateDefaultButtonClick(TObject *Sender)
{
Msg("Creating DirectInput Interface");

if (Dx_Input1->Created)
    Dx_Input1->Destroy();

Dx_Input1->Create();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Create(TObject *Sender)
{
Msg("DirectInput Interface has been created");
CreateDefaultButton->Enabled = false;
EnumerateButton->Enabled = true;
CreateSelectedButton->Enabled = false;
DestroyInterfaceButton->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (DevicesListBox->Items->Count>0)
    {
    int index = DevicesListBox->ItemIndex;
    AnsiString device_name = DevicesListBox->Items->Strings[index];
    Msg("Device Selected: '"+device_name+"'");

    CreateSelectedButton->Enabled = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateSelectedButtonClick(TObject *Sender)
{
Msg("Creating Selected Device");

if (Dx_InputDevice1->Created)
    Dx_InputDevice1->Destroy();

int index = DevicesListBox->ItemIndex;
if (DevicesListBox->Items->Strings[index] == "Keyboard") Dx_InputDevice1->Create( GUID_SysKeyboard, Dx_Input1 );
else if (DevicesListBox->Items->Strings[index] == "Mouse") Dx_InputDevice1->Create( GUID_SysMouse, Dx_Input1 );
else Dx_Input1->CreateDevice( InstanceGuids[index], Dx_InputDevice1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Input1Destroy(TObject *Sender)
{
Msg("DirectInput interface is about to be destroyed." );
if (Dx_InputDevice1->Created) Dx_InputDevice1->Destroy();
DestroyInterfaceButton->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1Create(TObject *Sender)
{
Msg("Input Device has been Created");
RetrieveCapsButton->Enabled = true;
DestroyActiveButton->Enabled = true;

Dx_InputDevice1->SetCooperativeLevel( Application->Handle, DISCL_EXCLUSIVE | DISCL_FOREGROUND );
Dx_InputDevice1->SetDataFormat( DIJoyState21 );

DIPropHeader1->Obj = 0;
DIPropHeader1->How = DIPH_DEVICE;
DIPropDWORD1->PropHeader = DIPropHeader1;
DIPropDWORD1->Data = 0;
Dx_InputDevice1->SetProperty( DIPROP_AUTOCENTER, DIPropDWORD1 );

Dx_InputDevice1->Acquire();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1Destroy(TObject *Sender)
{
Msg("Input Device is about to be Destroyed");
CreateSelectedButton->Enabled = true;
RetrieveCapsButton->Enabled = false;
DestroyActiveButton->Enabled = false;

Dx_InputDevice1->Unacquire();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RetrieveCapsButtonClick(TObject *Sender)
{
Msg("Retrieving Device Capabilities");
DeviceCapsListBox->Items->Clear();
if (Dx_InputDevice1->GetCapabilities( DIDevCaps1 ))
    {
    DeviceCapsListBox->Items->Add( "Axis: "+IntToStr( (int) DIDevCaps1->Axes) );
    DeviceCapsListBox->Items->Add( "Buttons: "+IntToStr( (int) DIDevCaps1->Buttons) );
    DeviceCapsListBox->Items->Add( "Buttons: "+IntToStr( (int) DIDevCaps1->Buttons) );
    DeviceCapsListBox->Items->Add( "FFDriverVersion: "+IntToStr( (int) DIDevCaps1->FFDriverVersion) );
    DeviceCapsListBox->Items->Add( "FFMinTimeResolution: "+IntToStr( (int) DIDevCaps1->FFMinTimeResolution) );
    DeviceCapsListBox->Items->Add( "FFSamplePeriod: "+IntToStr( (int) DIDevCaps1->FFSamplePeriod) );
    DeviceCapsListBox->Items->Add( "FirmwareRevision: "+IntToStr( (int) DIDevCaps1->FirmwareRevision) );
    DeviceCapsListBox->Items->Add( "Flags:" );
    DeviceCapsListBox->Items->AddStrings( DIDevCaps1->Flags_Strings );
    DeviceCapsListBox->Items->Add( "HardwareRevision: "+IntToStr( (int) DIDevCaps1->HardwareRevision) );
    DeviceCapsListBox->Items->Add( "POV's: "+IntToStr( (int) DIDevCaps1->POVs) );
    }
else
    {
    DeviceCapsListBox->Items->Add( "------ Unknown ------" );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyActiveButtonClick(TObject *Sender)
{
Msg("Destroying Active Device");
if (Dx_InputDevice1->Created) Dx_InputDevice1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyInterfaceButtonClick(TObject *Sender)
{
if (Dx_Input1->Created) Dx_Input1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_InputDevice1EnumEffects(TObject *Sender, TDIEffectInfo *EffectInfo, void *Context,
    bool &Finished)
{
if (NumEffects<MAX_EFFECTS)
    {
    EffectTypes[NumEffects] = EffectInfo->EffectType;
    EffectGuids[NumEffects] = EffectInfo->Guid;
    EffectNames[NumEffects] = EffectInfo->Name;
    EffectStaticParams[NumEffects] = EffectInfo->StaticParams;
    EffectDynamicParams[NumEffects] = EffectInfo->DynamicParams;

    EffectsListBox->Items->Add( EffectInfo->Name );

    NumEffects++;
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateEffectButtonClick(TObject *Sender)
{
if (EffectsListBox->Items->Count<1) return;
if (EffectsListBox->ItemIndex==-1) EffectsListBox->ItemIndex = 0;

dword   axes[2] = { DIJOFS_X, DIJOFS_Y };
long    direction[2] = { 0, 0 };

DIPeriodic1->Magnitude = DI_FFNOMINALMAX;
DIPeriodic1->Offset = 0;
DIPeriodic1->Phase = 0;
DIPeriodic1->Period = (DWORD) (0.05 * DI_SECONDS);

DIEnvelope1->AttackLevel = 0;
DIEnvelope1->AttackTime = (DWORD) (0.5 * DI_SECONDS);
DIEnvelope1->FadeLevel = 0;
DIEnvelope1->FadeTime = (DWORD) (1.0 * DI_SECONDS);

DIEffect1->Flags = DIEFF_POLAR | DIEFF_OBJECTOFFSETS;
DIEffect1->Duration = (DWORD) (2 * DI_SECONDS);

DIEffect1->SamplePeriod = 0;               // = default
DIEffect1->Gain = DI_FFNOMINALMAX;         // no scaling
DIEffect1->TriggerButton = DIJOFS_BUTTON0;
DIEffect1->TriggerRepeatInterval = 0;
DIEffect1->AxesCount = 2;
DIEffect1->Axes = &axes[0];
DIEffect1->Direction = &direction[0];
DIEffect1->Envelope = DIEnvelope1;
DIEffect1->TypeSpecificParams = DIPeriodic1;
DIEffect1->TypeSpecificParamsSize = DIPeriodic1->Size;

Dx_InputEffect1->Create( EffectGuids[EffectsListBox->ItemIndex], DIEffect1, Dx_InputDevice1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumEffectsButtonClick(TObject *Sender)
{
NumEffects = 0;
EffectsListBox->Items->Clear();
Dx_InputDevice1->EnumEffects( NULL, DIEFT_PERIODIC );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PlayEffectButtonClick(TObject *Sender)
{
DIEffect1->TriggerButton = DIEB_NOTRIGGER;
DIEffect1->Duration = INFINITE;
Dx_InputEffect1->Start( 1, 0 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopEffectButtonClick(TObject *Sender)
{
Dx_InputEffect1->Stop();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EffectsListBoxClick(TObject *Sender)
{
//    
}
//---------------------------------------------------------------------------
